"""Constants for the Rainforest Eagle integration."""

DOMAIN = "rainforest_eagle"
CONF_CLOUD_ID = "cloud_id"
CONF_INSTALL_CODE = "install_code"
CONF_HARDWARE_ADDRESS = "hardware_address"

TYPE_EAGLE_100 = "eagle-100"
TYPE_EAGLE_200 = "eagle-200"
