"""Define constants for the Slide component."""

from homeassistant.const import Platform

API = "api"
COMPONENT_PLATFORM = Platform.COVER
CONF_INVERT_POSITION = "invert_position"
DOMAIN = "slide"
SLIDES = "slides"
DEFAULT_OFFSET = 0.15
DEFAULT_RETRY = 120
