"""Constants for the Flick Electric integration."""

DOMAIN = "flick_electric"

CONF_TOKEN_EXPIRY = "expires"

ATTR_START_AT = "start_at"
ATTR_END_AT = "end_at"

ATTR_COMPONENTS = ["retailer", "ea", "metering", "generation", "admin", "network"]
