"""Constants for the Bluetooth Tracker component."""
from typing import Final

DOMAIN: Final = "bluetooth_tracker"
SERVICE_UPDATE: Final = "update"

BT_PREFIX: Final = "BT_"
CONF_REQUEST_RSSI: Final = "request_rssi"
DEFAULT_DEVICE_ID: Final = -1
